@@ -4,6 +4,7 @@ Defines the EasyXmlTree class, which is a convenience wrapper around etree.
 The class exposes some helpful functions like css_select() and xpath().
 """
 
+from html import unescape
 from typing import Dict, List, Union
 import unicodedata
 
@@ -96,12 +97,14 @@ class EasyXmlTree:
 		If return_string is true, return a single string value instead of a list.
 		"""
 
-		result: List[Union[str, EasyXmlElement]] = []
+		result: List[Union[str, EasyXmlElement, float]] = []
 
 		try:
 			query_result = self.etree.xpath(selector, namespaces=self.namespaces)
 			if isinstance(query_result, etree._ElementUnicodeResult): # pylint: disable=protected-access
 				result.append(str(query_result))
+			elif isinstance(query_result, float):
+				result.append(query_result)
 			else:
 				for element in query_result:
 					if isinstance(element, etree._ElementUnicodeResult): # pylint: disable=protected-access
@@ -370,11 +373,13 @@ class EasyXmlElement:
 		Shortcut to select elements based on xpath selector.
 		"""
 
-		result: List[Union[str, EasyXmlElement]] = []
+		result: List[Union[str, EasyXmlElement, float]] = []
 
 		query_result = self.lxml_element.xpath(selector, namespaces=self.namespaces)
 		if isinstance(query_result, etree._ElementUnicodeResult): # pylint: disable=protected-access
 			result.append(str(query_result))
+		elif isinstance(query_result, float):
+			result.append(query_result)
 		else:
 			for element in query_result:
 				if isinstance(element, etree._ElementUnicodeResult): # pylint: disable=protected-access
@@ -417,7 +422,7 @@ class EasyXmlElement:
 		`<p>Hello there, <abbr>Mr.</abbr> Smith!</p>` -> `Hello there, Mr. Smith!`
 		"""
 
-		return regex.sub(r"<[^>]+?>", "", self.inner_xml().strip())
+		return unescape(regex.sub(r"<[^>]+?>", "", self.inner_xml().strip()))
 
 	def remove(self) -> None:
 		"""
@@ -565,7 +570,7 @@ class EasyXmlElement:
 		children = []
 
 		for child in self.lxml_element:
-			children.append(EasyXmlElement(child))
+			children.append(EasyXmlElement(child, self.namespaces))
 
 		return children
 
