@@ -3,19 +3,16 @@
 Defines several functions that are useful for interacting with epub files.
 """
 
-import os
 from pathlib import Path
 import zipfile
-import itertools
-import regex
 from lxml import etree
 import se
 import se.easy_xml
 
 
-def convert_toc_to_ncx(epub_root_absolute_path: Path, toc_filename: str, xsl_filename: Path) -> se.easy_xml.EasyXhtmlTree:
+def convert_toc_to_ncx(epub_root_absolute_path: Path, toc_filename: str, xsl_filename: Path) -> se.easy_xml.EasyXmlTree:
 	"""
-	Take an epub3 HTML5 ToC file and convert it to an epub2 NCX file. NCX output is written to the same directory as the ToC file, in a file named "toc.ncx".
+	Take an HTML5 ToC file and convert it to an NCX file for compatibility with older ereaders. NCX output is written to the same directory as the ToC file, in a file named "toc.ncx".
 
 	epub structure must be in the SE format.
 
@@ -25,30 +22,33 @@ def convert_toc_to_ncx(epub_root_absolute_path: Path, toc_filename: str, xsl_fil
 	xsl_filename: The filename for the XSL file used to perform the transformation
 
 	OUTPUTS
-	An se.easy_xml.EasyXhtmlTree representing the HTML5 ToC file
+	An se.easy_xml.EasyXmlTree representing the HTML5 ToC file
 	"""
 
 	# Use an XSLT transform to generate the NCX
 	with open(epub_root_absolute_path / "epub" / toc_filename, "r", encoding="utf-8") as file:
 		xhtml = file.read()
 
-	toc_tree = se.easy_xml.EasyXhtmlTree(xhtml)
+	toc_tree = se.easy_xml.EasyXmlTree(xhtml)
 	transform = etree.XSLT(etree.parse(str(xsl_filename)))
-	ncx_tree = transform(etree.fromstring(str.encode(xhtml)), cwd=f"'{epub_root_absolute_path}{os.path.sep}'")
+	ncx_dom = se.easy_xml.EasyXmlTree(transform(etree.fromstring(str.encode(xhtml)), cwd=f"'{epub_root_absolute_path.as_posix()}/'"))
+
+	# Remove empty lang tags
+	for node in ncx_dom.xpath("//*[@xml:lang and re:test(@xml:lang, '^\\s*$')]"):
+		node.remove_attr("xml:lang")
+
+	for node in ncx_dom.xpath("//navMap"):
+		node.set_attr("id", "navmap")
+
+	# Make nicely incrementing navpoint IDs and playOrders
+	count = 1
+	for node in ncx_dom.xpath("//navPoint"):
+		node.set_attr("id", f"navpoint-{count}")
+		node.set_attr("playOrder", f"{count}")
+		count = count + 1
 
 	with open(epub_root_absolute_path / "epub" / "toc.ncx", "w", encoding="utf-8") as file:
-		ncx_xhtml = etree.tostring(ncx_tree, encoding="unicode", pretty_print=True, with_tail=False)
-		ncx_xhtml = regex.sub(r" xml:lang=\"\?\?\"", "", ncx_xhtml)
-
-		# Make nicely incrementing navpoint IDs and playOrders
-		ncx_xhtml = regex.sub(r"<navMap id=\".*\">", "<navMap id=\"navmap\">", ncx_xhtml)
-
-		counter = itertools.count(1)
-		ncx_xhtml = regex.sub(r"<navPoint id=\"id[\p{Letter}0-9]+?\"", lambda x: "<navPoint id=\"navpoint-{count}\" playOrder=\"{count}\"".format(count=next(counter)), ncx_xhtml)
-
-		ncx_xhtml = f"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n{ncx_xhtml}"
-
-		file.write(ncx_xhtml)
+		file.write(ncx_dom.to_string())
 
 	return toc_tree
 
@@ -69,7 +69,6 @@ def write_epub(epub_root_absolute_path: Path, output_absolute_path: Path) -> Non
 		epub.write(epub_root_absolute_path / "mimetype", "mimetype")
 		epub.write(epub_root_absolute_path / "META-INF" / "container.xml", "META-INF/container.xml", compress_type=zipfile.ZIP_DEFLATED)
 
-		for root, _, files in os.walk(epub_root_absolute_path):
-			for file in files:
-				if file not in ("mimetype", "container.xml"):
-					epub.write(Path(root) / file, (Path(root) / file).relative_to(epub_root_absolute_path), compress_type=zipfile.ZIP_DEFLATED)
+		for file_path in epub_root_absolute_path.glob("**/*"):
+			if file_path.name not in ("mimetype", "container.xml"):
+				epub.write(file_path, file_path.relative_to(epub_root_absolute_path), compress_type=zipfile.ZIP_DEFLATED)
