@@ -10,7 +10,7 @@ import math
 import string
 import unicodedata
 from pathlib import Path
-from typing import Union
+from typing import Dict, Union, List, Tuple
 
 import regex
 import roman
@@ -20,7 +20,7 @@ from titlecase import titlecase as pip_titlecase
 from unidecode import unidecode
 
 import se
-from se.easy_xml import EasyXmlTree
+from se.easy_xml import EasyXmlTree, EasyXmlElement
 
 
 # This list of phrasing tags is not intended to be exhaustive. The list is only used
@@ -368,7 +368,7 @@ def get_word_count(xhtml: str) -> int:
 def _replace_character_references(match_object) -> str:
 	"""Replace most XML character references with literal characters.
 
-	This function excludes &, >, and < (&amp;, &lt;, and &gt;), since
+	This function excludes ", ', &, >, and < (&amp;, &lt;, and &gt;), since
 	un-escaping them would create an invalid document.
 	"""
 
@@ -376,9 +376,9 @@ def _replace_character_references(match_object) -> str:
 
 	retval = entity
 
-	# Explicitly whitelist the three (nine) essential character references
+	# Explicitly whitelist the six (nine) essential character references
 	try:
-		if entity in ["&gt;", "&lt;", "&amp;", "&#62;", "&#60;", "&#38;", "&#x3e;", "&#x3c;", "&#x26;"]:
+		if entity in ["&gt;", "&lt;", "&amp;", "&quot;", "&apos;", "&#62;", "&#60;", "&#38;", "&#x3e;", "&#x3c;", "&#x26;"]:
 			retval = entity
 		# Convert base 16 references
 		elif entity.startswith("&#x"):
@@ -1169,7 +1169,7 @@ def titlecase(text: str) -> str:
 	text = regex.sub(r"\b Th’ \b", " th’ ", text)
 
 	# Lowercase `o’`
-	text = regex.sub(r"\b O’ \b", " o’ ", text)
+	text = regex.sub(r" O’ ", " o’ ", text)
 
 	# Uppercase words that begin compound words, like `to-night` (which might appear in poetry)
 	text = regex.sub(r" ([\p{Lowercase_Letter}])([\p{Lowercase_Letter}]+\-)", lambda result: " " + result.group(1).upper() + result.group(2), text)
@@ -1475,3 +1475,129 @@ def generate_title(xhtml: Union[str, EasyXmlTree]) -> str:
 	title = title.replace("&amp;", "&")
 
 	return title
+
+def _get_flattened_children(node: EasyXmlElement, allow_header: bool) -> List[EasyXmlElement]:
+	"""
+	Helper function for find_unexpected_ids().
+
+	Get a flat list of children that are not headers or sectioning elements,
+	and return a list of those nodes.
+	In other words, input like this:
+	<section>
+		<p></p>
+		<blockquote>
+			<p></p>
+		</blockquote>
+		<p></p>
+	</section>
+
+	Would result in this flat list:
+	p
+	blockquote
+	p
+	p
+	"""
+
+	result = []
+	sectioning_elements = ["section", "article", "figure", "nav"]
+
+	if not allow_header:
+		sectioning_elements.append("header")
+
+	for child in node.children:
+		is_endnote = False
+		is_glossdef = False
+		if child.get_attr("epub:type"):
+			is_endnote = regex.search(r"\bendnote\b", child.get_attr("epub:type"))
+			is_glossdef = "glossdef" in child.get_attr("epub:type")
+
+		if child.tag not in sectioning_elements and not is_endnote and not is_glossdef:
+			result.append(child)
+			result = result + _get_flattened_children(child, allow_header)
+
+	return result
+
+def find_unexpected_ids(dom: EasyXmlTree) -> List[Tuple[EasyXmlElement, str]]:
+	"""
+	Given a DOM tree, return a list of tuples containing nodes and their expected ID attributes.
+	Only nodes with unexpected IDs are returned.
+
+	INPUTS
+	dom: An EasyXmlTree
+
+	OUTPUTS
+	A list of tuples of (node, expected_id)
+	"""
+
+	dom_copy = deepcopy(dom)
+	replacements = []
+
+	# Remove noterefs as they have their own ID rules
+	for node in dom_copy.xpath("/html/body//*[contains(@epub:type, 'noteref')]"):
+		node.remove()
+
+	# IDs are set to `{closest_parent_sectioning_element_id}-{tag_name}-{n}`.
+	# Lines of poetry are set to `{closest_poem_sectioning_element_id}-line-{n}`.
+	# Notes:
+	# 1. Endnotes count as their own sectioning elements for the purposes of assigning IDs
+	# 2. Exclude glossaries, as IDs there should be related to the glossterm somehow
+	line_number = 0
+	endnote_number = 0
+	container_poem_section_id = ""
+	for section in dom_copy.xpath("/html/body//*[@id and (name() = 'section' or name() = 'article' or re:test(@epub:type, '\\bendnote\\b'))]"):
+		counts: Dict[str, int] = {}
+		is_poem = bool(section.xpath("./ancestor-or-self::*[contains(@epub:type, 'z3998:poem')]"))
+		section_id = section.get_attr("id")
+		allow_header = not is_poem
+
+		section_epub_type = section.get_attr("epub:type")
+		if section_epub_type:
+			# If this section is a poem or an endnotes container, reset the counters
+			if "z3998:poem" in section_epub_type:
+				line_number = 0
+				container_poem_section_id = section_id
+				allow_header = False
+
+			if "endnotes" in section_epub_type:
+				endnote_number = 0
+
+			# If this section is an endnote, increment the note number and check the ID right now
+			if regex.search(r"\bendnote\b", section.get_attr("epub:type")):
+				endnote_number = endnote_number + 1
+				expected_id = f"note-{endnote_number}"
+
+				if section_id != expected_id:
+					replacements.append((section, expected_id))
+
+		for node in _get_flattened_children(section, allow_header):
+			if node.tag in counts:
+				counts[node.tag] = counts[node.tag] + 1
+			else:
+				counts[node.tag] = 1
+
+			# If the line is a line of poetry, increment the line count
+			if is_poem and node.tag == "span" and node.parent.tag == "p":
+				line_number = line_number + 1
+
+			id_attr = node.get_attr("id")
+			# If the element has an ID attribute and it's not an endnote node (i.e. <li epub:type="endnote">)
+			if id_attr:
+				expected_id = id_attr
+
+				# <dt>s have an id attribute set to their defining word
+				if node.tag == "dt":
+					dfn = node.xpath("./dfn")
+					if dfn:
+						expected_id = make_url_safe(dfn[0].inner_text())
+
+				# All other elements
+				else:
+					expected_id = f"{section_id}-{node.tag}-{counts[node.tag]}"
+
+					if is_poem and node.tag == "span" and node.parent.tag == "p":
+						expected_id = f"{container_poem_section_id}-line-{line_number}"
+
+				if id_attr != expected_id:
+					replacements.append((node, expected_id))
+
+	return replacements
