@@ -47,6 +47,16 @@ class Endnote:
 		self.source_file = ""
 		self.matched = False
 
+class EndnoteChange:
+	"""
+	Class to hold a record of what changes have been made to endnote numbers
+	"""
+
+	def __init__(self, old_anchor, new_anchor, filename):
+		self.old_anchor = old_anchor  # the previous anchor
+		self.new_anchor = new_anchor  # the anchor it has been changed to
+		self.filename = filename	# the file in which it was changed
+
 class SeEpub:
 	"""
 	An object representing an SE epub file.
@@ -59,6 +69,7 @@ class SeEpub:
 	content_path: Path = Path() # The path to the epub content base, i.e. self.epub_root_path / epub
 	metadata_file_path: Path = Path() # The path to the metadata file, i.e. self.content_path / content.opf
 	toc_path: Path = Path()  # The path to the metadata file, i.e. self.content_path / toc.xhtml
+	glossary_search_key_map_path = None # The path to the glossary search key map, or None
 	local_css = ""
 	is_se_ebook = True
 	_file_cache: Dict[str, str] = {}
@@ -110,6 +121,10 @@ class SeEpub:
 		else:
 			raise se.InvalidSeEbookException("Couldn’t find table of contents.")
 
+		gskm_href = self.metadata_dom.xpath("/package/manifest/item[contains(@properties, 'search-key-map')]/@href", True)
+		if gskm_href:
+			self.glossary_search_key_map_path = self.content_path / gskm_href
+
 		# If our identifier isn't SE-style, we're not an SE ebook
 		identifier = self.metadata_dom.xpath("/package/metadata/dc:identifier/text()", True)
 		if not identifier or not identifier.startswith("url:https://standardebooks.org/ebooks/"):
@@ -464,18 +479,7 @@ class SeEpub:
 		# We even convert SVGs instead of inlining them, because CSS won't allow us to style inlined SVGs
 		# (for example if we want to apply max-width or filter: invert())
 		for img in section.xpath("//img[starts-with(@src, '../images/')]"):
-			src = img.get_attr("src").replace("../", "")
-			with open(self.content_path / src, "rb") as binary_file:
-				image_contents_base64 = base64.b64encode(binary_file.read()).decode()
-
-			if src.endswith(".svg"):
-				img.set_attr("src", f"data:image/svg+xml;base64,{image_contents_base64}")
-
-			if src.endswith(".jpg"):
-				img.set_attr("src", f"data:image/jpg;base64,{image_contents_base64}")
-
-			if src.endswith(".png"):
-				img.set_attr("src", f"data:image/png;base64,{image_contents_base64}")
+			img.set_attr("src", se.images.get_data_url(self.content_path / img.get_attr("src").replace("../", "")))
 
 	def recompose(self, output_xhtml5: bool, extra_css_file: Path = None) -> str:
 		"""
@@ -494,37 +498,42 @@ class SeEpub:
 		css = ""
 		namespaces: List[str] = []
 
-		css_filenames = ["core.css", "se.css", "local.css"]
+		css_filenames = list(self.content_path.glob("**/*.css"))
 
+		# If we have standard SE CSS files, attempt to sort them in the order we expect them to appear in the ebook.
+		# core.css -> se.css -> local.css
+		sorted_css_filenames = {}
+		for filepath in css_filenames:
+			if filepath.name == "core.css":
+				sorted_css_filenames[0] = filepath
+			if filepath.name == "se.css":
+				sorted_css_filenames[1] = filepath
+			if filepath.name == "local.css":
+				sorted_css_filenames[2] = filepath
+
+		# Turn the dict into a list, sorting by key
+		sorted_css_filenames_list = [sorted_css_filenames[key] for key in sorted(sorted_css_filenames.keys())]
+
+		# If we hit all 3 files then we're probably an SE ebook, replace our list of CSS files with the sorted list
+		if len(sorted_css_filenames_list) == 3:
+			css_filenames = sorted_css_filenames_list
+
+		# Add the extra CSS file if present
 		if extra_css_file:
-			css_filenames.append(str(extra_css_file))
+			css_filenames.append(extra_css_file)
 
-		for filename in css_filenames:
-			filepath = self.content_path / "css" / filename
+		# Now recompose the CSS
+		for filepath in css_filenames:
 			file_css = self.get_file(filepath)
 
 			namespaces = namespaces + regex.findall(r"@namespace.+?;", file_css)
 
 			file_css = regex.sub(r"\s*@(charset|namespace).+?;\s*", "\n", file_css).strip()
 
-			# Convert background-image URLs (from local.css only) to base64
-			if filename == "local.css":
-				for image in regex.finditer(pattern=r"""url\("(.+?\.(?:svg|png|jpg))"\)""", string=file_css):
-					image_path = image.captures(1)[0].replace("../", "")
-
-					with open(self.content_path / image_path, "rb") as binary_file:
-						image_contents_base64 = base64.b64encode(binary_file.read()).decode()
-
-					if image_path.endswith(".svg"):
-						replacement = f"""url("data:image/svg+xml;base64,{image_contents_base64}")"""
-					elif image_path.endswith(".jpg"):
-						replacement = f"""url("data:image/jpg;base64,{image_contents_base64}")"""
-					elif image_path.endswith(".png"):
-						replacement = f"""url("data:image/png;base64,{image_contents_base64}")"""
-					else:
-						continue
-
-					file_css = file_css.replace(image.group(0), replacement)
+			# Convert background-image URLs to base64
+			for image in regex.finditer(pattern=r"""url\("(.+?\.(?:svg|png|jpg))"\)""", string=file_css):
+				data_url = se.images.get_data_url(self.content_path / image.captures(1)[0].replace("../", ""))
+				file_css = file_css.replace(image.group(0), f"""url("{data_url}")""")
 
 			css = css + f"\n\n\n/* {filepath.name} */\n" + file_css
 
@@ -947,7 +956,6 @@ class SeEpub:
 				# the `glossary` semantic may also appear in the ToC landmarks, so specifically exclude that
 				if dom.xpath("//*[contains(@epub:type, 'glossary') and not(ancestor-or-self::nav)]"):
 					properties.append("glossary")
-					#if dom.xpath("/html/body//*[namespace-uri()='http://www.w3.org/1998/Math/MathML']"):
 				if dom.xpath("/html[namespace::*='http://www.w3.org/1998/Math/MathML']"):
 					properties.append("mathml")
 
@@ -1193,7 +1201,7 @@ class SeEpub:
 				anchor = ""
 				href = link.get_attr("href") or ""
 				if href:
-					# Extract just the anchor from a URL (ie, what follows a hash symbol)
+					# Extract just the anchor from a URL (i.e., what follows a hash symbol)
 					hash_position = href.find("#") + 1  # we want the characters AFTER the hash
 					if hash_position > 0:
 						anchor = href[hash_position:]
@@ -1216,6 +1224,8 @@ class SeEpub:
 				orphans.append(note.anchor)
 		for orphan in orphans:
 			response.append(f"Orphan endnote with anchor: {orphan}")
+		if len(orphans) > 0:
+			response.append("Is your spine generated and valid?")
 		return response
 
 	def recreate_endnotes(self) -> None:
@@ -1263,20 +1273,20 @@ class SeEpub:
 		Read the epub spine to regenerate all endnotes in order of appearance, starting from 1.
 		Changes are written to disk.
 
-		Returns a tuple of (found_endnote_count, changed_endnote_count)
+		Returns a tuple of (found_endnote_count, changed_endnote_count, change_list)
 		"""
 
 		# Do a safety check first, throw exception if it failed
 		results = self._check_endnotes()
 		if results:
 			report = "\n".join(results)
-			raise se.InvalidInputException(f"Endnote error(s) found: {report}.")
+			raise se.InvalidInputException(f"Endnote error(s) found:\n{report}")
 
 		# If we get here, it's safe to proceed
 		processed = 0
 		current_note_number = 1
 		notes_changed = 0
-		change_list: List[str] = []
+		change_list: List[EndnoteChange] = []
 
 		for file_path in self.spine_file_paths:
 			dom = self.get_dom(file_path)
@@ -1289,7 +1299,7 @@ class SeEpub:
 
 			needs_rewrite = False
 			for link in dom.xpath("/html/body//a[contains(@epub:type, 'noteref')]"):
-				needs_rewrite, notes_changed = self.__process_link(change_list, current_note_number, file_path.name, link, needs_rewrite, notes_changed)
+				needs_rewrite, notes_changed = self.__process_noteref_link(change_list, current_note_number, file_path.name, link, needs_rewrite, notes_changed)
 				current_note_number += 1
 
 			# If we need to write back the body text file
@@ -1302,7 +1312,7 @@ class SeEpub:
 			node = source_note.node
 			needs_rewrite = False
 			for link in node.xpath(".//a[contains(@epub:type, 'noteref')]"):
-				needs_rewrite, notes_changed = self.__process_link(change_list, current_note_number, self.endnotes_path.name, link, needs_rewrite, notes_changed)
+				needs_rewrite, notes_changed = self.__process_noteref_link(change_list, current_note_number, self.endnotes_path.name, link, needs_rewrite, notes_changed)
 				current_note_number += 1
 
 		if processed == 0:
@@ -1329,9 +1339,43 @@ class SeEpub:
 			with open(self.endnotes_path, "w") as file:
 				file.write(se.formatting.format_xhtml(endnotes_dom.to_string()))
 
+			# now trawl through the body files to locate any direct links to endnotes (not in an actual endnote reference)
+			# example: (see <a href="endnotes.xhtml#note-1553">this note</a>.)
+			# most but not all such are likely to be in the newly re-written endnotes.xhtml
+			for file_path in self.spine_file_paths:
+				needs_rewrite = False
+				dom = self.get_dom(file_path)
+				for link in dom.xpath("/html/body//a[contains(@href, 'endnotes.xhtml#note-')]"):
+					needs_rewrite = self.__process_direct_link(change_list, link)
+				if needs_rewrite:
+					with open(file_path, "w") as file:
+						file.write(se.formatting.format_xhtml(dom.to_string()))
+
 		return current_note_number - 1, notes_changed, change_list
 
-	def __process_link(self, change_list, current_note_number, file_name, link, needs_rewrite, notes_changed) -> Tuple[bool, int]:
+	def __process_direct_link(self, change_list, link) -> bool:
+		"""
+		Checks all hyperlinks to the endnotes to see if the existing anchor needs to be updated with a new number
+
+		Returns a boolean of needs_write (whether object needs to be re-written)
+		"""
+		epub_type = link.get_attr("epub:type") or ""
+		if not epub_type: # it wasn't an actual endnote reference but a direct link (we hope!)
+			href = link.get_attr("href") or ""
+			if href:
+				# Extract just the anchor from a URL (ie, what follows a hash symbol)
+				hash_position = href.find("#") + 1  # we want the characters AFTER the hash
+				if hash_position > 0:
+					old_anchor = href[hash_position:]
+					try:
+						change = next(ch for ch in change_list if ch.old_anchor == old_anchor)
+						link.set_attr("href", f"{self.endnotes_path.name}#{change.new_anchor}")
+						return True
+					except StopIteration:  # didn't find the old anchor, keep going
+						pass
+		return False
+
+	def __process_noteref_link(self, change_list, current_note_number, file_name, link, needs_rewrite, notes_changed) -> Tuple[bool, int]:
 		"""
 		Checks each endnote link to see if the existing anchor needs to be updated with a new number
 
@@ -1348,7 +1392,8 @@ class SeEpub:
 
 		new_anchor = f"note-{current_note_number:d}"
 		if new_anchor != old_anchor:
-			change_list.append(f"{old_anchor}->{new_anchor}, {file_name}")
+			endnote_change = EndnoteChange(old_anchor, new_anchor, file_name)
+			change_list.append(endnote_change)
 			notes_changed += 1
 			# Update the link in the dom
 			link.set_attr("href", f"{self.endnotes_path.name}#{new_anchor}")
