@@ -346,6 +346,7 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b([Aa])ny thing\b", r"\1nything", xhtml)			# any thing -> anything
 	xhtml = regex.sub(r"\b([Rr])e-?enforce", r"\1einforce", xhtml)			# re-enforce -> reinforce
 	xhtml = regex.sub(r"\b([Ll])uny", r"\1oony", xhtml)				# luny -> loony
+	xhtml = regex.sub(r"\b([Ll])unies", r"\1oonies", xhtml)				# lunies -> loonies
 	xhtml = regex.sub(r"\b([Vv])icuna", r"\1icuña", xhtml)				# vicuna -> vicuña
 	xhtml = regex.sub(r"\b([Cc])larionet", r"\1larinet", xhtml)			# clarionet -> clarinet
 	xhtml = regex.sub(r"\b([Bb])ye?[\- ]the[\- ]bye?\b", r"\1y the by", xhtml)	# by-the-bye -> by the by
@@ -379,15 +380,36 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b[‘’]([Ss])cope(s?)\b", r"\1cope\2", xhtml)		# 'scope -> scope
 	xhtml = regex.sub(r"\b([Ff])(aqu?ir|akeer)", r"\1akir", xhtml)			# faqir, fakeer -> fakir
 	xhtml = regex.sub(r"\b([Ii])maum", r"\1mam", xhtml)				# imaum -> imam
-	xhtml = regex.sub(r"\b([Mm])oujik", r"\1uzhik", xhtml)				# moujik -> muzhik
-	xhtml = regex.sub(r"\b([Cc])har[ -][aà][ -]banc", r"\1arabanc", xhtml)		# char-à-banc -> charabanc
+	xhtml = regex.sub(r"\b([Mm])o?ujik", r"\1uzhik", xhtml)				# moujik/mujik -> muzhik
+	xhtml = regex.sub(r"\b([Cc])har[ -][aà][ -]banc", r"\1harabanc", xhtml)		# char-à-banc -> charabanc
 	xhtml = regex.sub(r"’([Cc])ellist", r"\1ellist", xhtml)				# 'cellist -> cellist
 	xhtml = regex.sub(r"([Pp])ourtray", r"\1ortray", xhtml)				# pourtray -> portray
 	xhtml = regex.sub(r"([S])toopid", r"\1tupid", xhtml)				# stoopid -> stupid
 	xhtml = regex.sub(r"([S])uède", r"\1uede", xhtml)				# suède -> suede
 	xhtml = regex.sub(r"([Ff])or ever\b",r"\1orever", xhtml)			# for ever -> forever
 	xhtml = regex.sub(r"([Dd])ébris\b", r"\1ebris", xhtml)				# débris -> debris
-	xhtml = regex.sub(r"\b([Tt])ho['’]\b", r"\1hough", xhtml)			# tho' -> though
+	xhtml = regex.sub(r"\b([Tt])ho(['’]|\b)(?!\.</abbr>)", r"\1hough", xhtml)	# tho' -> though
+	xhtml = regex.sub(r"\b([Aa])ntient", r"\1ncient", xhtml)			# antient -> ancient
+	xhtml = regex.sub(r"\b([Bb])efal(s?)\b", r"\1efall\2", xhtml)			# befal -> befall
+	xhtml = regex.sub(r"\b([Tt])enour", r"\1enor", xhtml)				# tenor -> tenour
+	xhtml = regex.sub(r"\b([Rr])ibb?and", r"\1ibbon", xhtml)			# ribband -> ribbon (note: `ribband` has a legitimate 2nd meaning but it's extremely rare)
+	xhtml = regex.sub(r"\b([Gg])rewsome", r"\1ruesome", xhtml)			# grewsome -> gruesome
+	xhtml = regex.sub(r"([^’]\b[Tt])il\b", r"\1ill", xhtml)				# til -> till (but not 'til which can look more natural in dialect)
+	xhtml = regex.sub(r"\b([Cc])heer", r"\1heer", xhtml)				# chear -> cheer
+	xhtml = regex.sub(r"\bcentinel", r"sentinel", xhtml)				# centinel -> sentinel
+	xhtml = regex.sub(r"\bCentinel", r"Sentinel", xhtml)				# centinel -> sentinel
+	xhtml = regex.sub(r"\b([Ss])e’nnight", r"\1ennight", xhtml)			# se'nnight -> sennight
+	xhtml = regex.sub(r"\b([Pp])aroquet", r"\1arakeet", xhtml)			# paroquet -> parakeet
+	xhtml = regex.sub(r"\b([Rr])isque\b", r"\1isqué", xhtml)			# risque -> risqué
+	xhtml = regex.sub(r"\b([Ff])rolick\b", r"\1rolic", xhtml)			# frolick -> frolic
+	xhtml = regex.sub(r"\bIncroach", r"Encroach", xhtml)				# Incroach -> Encroach
+	xhtml = regex.sub(r"\bincroach", r"encroach", xhtml)				# incroach -> encroach
+	xhtml = regex.sub(r"\bmizen ?mast", r"mizzenmast", xhtml)			# mizenmast -> mizzenmast
+	xhtml = regex.sub(r"\bbefal(s?)\b", r"befall\1", xhtml)				# befal -> befall
+	xhtml = regex.sub(r"\bto a [‘“]?[Tt]([^’”]?)[’”]?(\s)", r"to a T\1\2", xhtml)		# Remove quotes from `to a T` and capitalize T
+	xhtml = regex.sub(r"\b([Tt])eaze", r"\1ease", xhtml)				# teaze -> tease
+	xhtml = regex.sub(r"\b([Cc])hrystal", r"\1rystal", xhtml)			# chrystal -> crystal
+	xhtml = regex.sub(r"\b([Pp])arty-?colo(u?)r", r"\1articolo\2r", xhtml)		# party-color -> particolor
 
 	# Normalize some names
 	xhtml = regex.sub(r"Moliere", r"Molière", xhtml)				# Moliere -> Molière
@@ -437,6 +459,7 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"Roman catholic", r"Roman Catholic", xhtml)			# Roman catholic -> Roman Catholic; Note that we can't uppercase `catholic` in the generic sense because `catholic` can mean "worldly"
 	xhtml = regex.sub(r"Burmah", r"Burma", xhtml)					# Burmah -> Burma
 	xhtml = regex.sub(r"Turgenieff", r"Turgenev", xhtml)				# Turgenieff -> Turgenev
+	xhtml = regex.sub(r"Gizeh", r"Giza", xhtml)					# Gizeh -> Giza
 
 	# Remove archaic diphthongs
 	xhtml = regex.sub(r"\b([Mm])edi(æ|ae)val", r"\1edieval", xhtml)
