@@ -76,7 +76,7 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 		xhtml = xhtml.replace("`", "'")
 
 		# First, convert entities.  Sometimes Gutenberg has entities instead of straight quotes.
-		xhtml = html.unescape(xhtml) # This converts html entites to unicode
+		xhtml = html.unescape(xhtml) # This converts html entities to unicode
 		xhtml = regex.sub(r"&(?![#\p{Lowercase_Letter}]+;)", "&amp;", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
 
 		# Replace rsquo character with an escape sequence. We can't use HTML comments
@@ -87,7 +87,7 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 		xhtml = smartypants.smartypants(xhtml) # Attr.u *should* output unicode characters instead of HTML entities, but it doesn't work
 
 		# Convert entities again
-		xhtml = html.unescape(xhtml) # This converts html entites to unicode
+		xhtml = html.unescape(xhtml) # This converts html entities to unicode
 		xhtml = regex.sub(r"&(?![#\p{Lowercase_Letter}]+;)", "&amp;", xhtml) # Oops!  html.unescape also unescapes plain ampersands...
 
 	# Replace no-break hyphen with regular hyphen
@@ -188,8 +188,12 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 
 	xhtml = regex.sub(r"c/o", "℅", xhtml, flags=regex.IGNORECASE)
 
-	# Add rsquo to bare `tis` and `twas`
-	xhtml = regex.sub(r"([^’>])\b([Tt]is|[Tt]was|[Tt]were|[Tt]won’t)\b", r"\1’\2", xhtml)
+	# Sort out preceding rsquos for  `tis` / `twas` / `twere` / `twont’t`
+	# 1. If there’s a missing quote (after a space or tag) add it.
+	# 2. If there’s a right double-quote assumed it should be an open and closing single quote pair.
+	contractions = r"[Tt]is|[Tt]was|[Tt]were|[Tt]won’t"
+	xhtml = regex.sub(fr"([\s>])({ contractions })\b", r"\1’\2", xhtml) # 1.
+	xhtml = regex.sub(fr"”\b({ contractions })\b", r"‘’\1", xhtml)      # 2.
 
 	# Replace `M‘<letter>` with `Mc<letter>`; use of lsquo in this case is a historical case of a "poor man's superscript c". See
 	# https://english.stackexchange.com/questions/543272/why-were-scottish-irish-names-once-rendered-with-apostrophes-instead-of-mac#543329
@@ -333,8 +337,11 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	# Add an &nbsp; before &amp;
 	xhtml = regex.sub(r" &amp;", f"{se.NO_BREAK_SPACE}&amp;", xhtml)
 
-	# Add word joines to ellipses
-	xhtml = regex.sub(r" …", f"{se.WORD_JOINER} {se.WORD_JOINER}…", xhtml)
+	# Add word joiners to ellipses
+	xhtml = regex.sub(fr"{se.HAIR_SPACE}…", f"{se.WORD_JOINER}{se.HAIR_SPACE}{se.WORD_JOINER}…", xhtml)
+
+	# Remove any extra spaces in 4-dot ellipses that occur before the starting period
+	xhtml = regex.sub(fr"([\p{{Letter}}]){se.HAIR_SPACE}(\.{se.WORD_JOINER}{se.HAIR_SPACE}{se.WORD_JOINER}…)", r"\1\2", xhtml)
 
 	# Remove spaces between ellipses and endnotes directly after
 	xhtml = regex.sub(fr"…[\s{se.NO_BREAK_SPACE}]?(<a[^>]+?epub:type=\"noteref\"[^>]*?>)", r"…\1", xhtml, flags=regex.IGNORECASE)
@@ -347,6 +354,9 @@ def typogrify(xhtml: str, smart_quotes: bool = True) -> str:
 	for match in regex.findall(fr"<title>[^<]*?[{se.NO_BREAK_SPACE}{se.WORD_JOINER}][^<]*?</title>", xhtml):
 		xhtml = xhtml.replace(match, match.replace(se.NO_BREAK_SPACE, " ").replace(se.WORD_JOINER, ""))
 
+	# Remove no-break spaces added before etc.
+	xhtml = regex.sub(fr"{se.NO_BREAK_SPACE}(<abbr[^>]*?>etc\.)", r" \1", xhtml)
+
 	return xhtml
 
 def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_tags: bool = False) -> str:
@@ -377,11 +387,11 @@ def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_
 		except Exception as ex:
 			raise se.InvalidLanguageException("No [attr]xml:lang[/] or [attr]lang[/] attribute on [xhtml]<html>[/] element; couldn’t guess file language.") from ex
 
-	language = language.replace("-", "_")
-
 	# Cope with known missing languages
-	if language in ["en_AU", "en_CA"] :
-		language = "en_GB"
+	if language in ["en-AU", "en-CA", "en-IE"] :
+		language = "en-GB"
+
+	language = language.replace("-", "_")
 
 	if language not in pyphen.LANGUAGES:
 		raise se.MissingDependencyException(f"Hyphenator for language [text]{language}[/] not available.\nInstalled hyphenators: {pyphen.LANGUAGES}.")
@@ -389,13 +399,12 @@ def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_
 	hyphenator = pyphen.Pyphen(lang=language)
 
 	text = dom.xpath("/html/body")[0].inner_xml()
-	result = text
+	result = ""
 	word = ""
 	in_tag = False
 	tag_name = ""
 	reading_tag_name = False
 	in_h_tag = False
-	pos = 1
 
 	# The general idea here is to read the whole contents of the <body> tag character by character.
 	# If we hit a <, we ignore the contents until we hit the next >.
@@ -403,6 +412,7 @@ def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_
 	# We can't just split at whitespace because HTML tags can contain whitespace (attributes for example)
 	for char in text:
 		process = False
+		reading_word = False
 
 		if char == "<":
 			process = True
@@ -419,6 +429,7 @@ def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_
 			tag_name = tag_name + char
 		elif not in_tag and char.isalnum():
 			word = word + char
+			reading_word = True
 		elif not in_tag:
 			process = True
 
@@ -435,11 +446,13 @@ def hyphenate(xhtml: Union[str, EasyXmlTree], language: Optional[str], ignore_h_
 		if process:
 			if word != "":
 				new_word = hyphenator.inserted(word, hyphen=se.SHY_HYPHEN)
-				result = result[:pos - len(word) - 1] + new_word + char + result[pos:]
-				pos = pos + len(new_word) - len(word)
-			word = ""
+				result = result + new_word + char
+			else:
+				result = result + char
 
-		pos = pos + 1
+			word = ""
+		elif not reading_word or (in_h_tag and ignore_h_tags):
+			result = result + char
 
 	# We need to double-escape backslashes in the replacement string, in case the string contains a backslash+number that
 	# the regex engine will misinterpret as a capture group
